# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::Atom;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::Config;

use constant WEBLOG => 0;
use constant ENTRY => 1;
use constant COMMENT => 2;


=item iso_date
@: date (as in localtime())
r: date as string
=cut
sub iso_date(@) {
	my @date = @_;
	return sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",
			$date[5] + 1900,	# YYYY
			$date[4] + 1,		# MM
			$date[3],		# DD
			$date[2],		# HH24
			$date[1],		# MI
			$date[0]);		# SS
}


=item id
Get feed item ID.

0: type: 0 = weblog, 1 = entry, 2 = comment
1: entry id
2: (comment id)
r: atom::id
=cut
sub id($$$) {
	my ($type, $entry, $comment) = @_;
	my $uri = $Woclema::Config::base_url;
	defined($uri) || die "Woclema::Config::base_url must be set " .
			"to enable Atom feed!";
	$uri =~ s|^\w+://||;
	my ($domain, $path) = split(/\//, $uri, 2);
	$path =~ s|/+|/|g;

	my $tag = "tag:$domain,2007:$path";
	($type == WEBLOG) && return $tag;

	$tag .= ",entry=$entry";
	$tag .= ",comment=$comment" if (defined($comment));
	return $tag;
}


=item print_entry
0: entry as hash
r: entry as xml
=cut
sub sprint_entry(%) {
	my %entry = %{(shift)};
	my $r;

	my $published = &iso_date(gmtime($entry{'published'}));
	my $updated;
	$updated = &iso_date(gmtime($entry{'updated'}))
		if defined($entry{'updated'});

	$r = "<entry>\n";
	$r .= "\t<title>" . $entry{'title'} . "</title>\n"
		if defined($entry{'title'});
	$r .= "\t<link href=\"" . $entry{'link'} . "\" />\n"
		if defined($entry{'link'});
	$r .= "\t<id>" . $entry{'id'} . "</id>\n";
	$r .= "\t<published>$published</published>\n";
	$r .= "\t<updated>$updated</updated>\n"
		if defined($updated);
	$r .= "\t<summary>" . $entry{'summary'} . "</summary>\n"
		if defined($entry{'summary'});
	if (defined($entry{'content'})) {
		if (defined($entry{'content-type'})) {
			$r .= sprintf("\t<content type=\"%s\">\n",
					$entry{'content-type'});
		} else {
			$r .= "\t<content>\n";
		}
		$r .= "<![CDATA[$entry{'content'}]]>";
		$r .= "\n\t</content>\n";
	}
	$r .= "</entry>\n";

	return $r;
}


sub sprint(\@@) {
	my @time = @{(shift)};
	my @entries = @_;

	my $id = &id(WEBLOG);
	my $updated = &iso_date(@time);
	my $author = "";
	if (defined($Woclema::Config::feed_author_name)) {
		$author = "<author>\n";
		$author .= "\t<name>$Woclema::Config::feed_author_name" .
			"</name>\n";
		$author .= "\t<email>$Woclema::Config::feed_author_email" .
			"</email>\n";
		$author .= "</author>";
	}
	my $title = "";
	if (defined($Woclema::Config::feed_title)) {
		$title = "<title type=\"text\">$Woclema::Config::feed_title" .
			"</title>";
	}

	my $self = "$Woclema::Config::base_url/atom.pl";
	$self =~ s/([^:])\/\/+/$1\//g;
	my $r = sprintf(<<EOF);
<?xml version="1.0" encoding="UTF-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">
<id>$id</id>
<updated>$updated</updated>
$title
<link rel="alternate" type="text/html" href="$Woclema::Config::base_url" />
<link rel="self" type="application/atom+xml" href="$self" />
$author
<generator uri="http://katei.fi/woclema.html">Woclema</generator>
EOF

	foreach (@entries) {
		my %entry = %{$_};
		$r .= &sprint_entry(\%entry);
	}

	$r .= "</feed>";
}


return 1;
