#! /usr/bin/perl
#
# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2009 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# wanha - your pesky URL-guardian for IRC


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::Atom;
use Woclema::Entry;

use Woclema::Link;
use Woclema::HTML;


=item main
Get Atom-feed of the weblog.
=cut
sub main() {
	my $latest = gmtime();
	my $n = $Woclema::Config::feed_entries;
	my @list;
	my $pub = 1;

	if ($#ARGV >= 0) {
		if ($ARGV[0] eq "unpub") {
			$pub = 0;
			shift(@ARGV);
		} elsif ($ARGV[0] eq "pub") {
			$pub = 1;	
			shift(@ARGV);
		}
	}
	if ($#ARGV == -1) {
		@list = &Woclema::Entry::get_list_before_time(
				$latest, $n, $pub);
	} else {
		foreach my $id (@ARGV) {
			my %e = &Woclema::Entry::get($id, ! $pub);
			push(@list, \%e);
		}
	}

	my @e_list;
	foreach (@list) {
		my %orig = %{$_};
		my %e = (
			"id"		=> &Woclema::Atom::id(
						Woclema::Atom::ENTRY,
						$orig{'id'}),
			"title" 	=> $orig{'title'},
			"content"	=> $orig{'text'},
			"link"		=> &Woclema::Link::entry($orig{'id'}),
			"published"	=> $orig{'time_epoch'},
			"updated"	=> $orig{'edit_epoch'}
			);
		$e{"updated"} = $e{"published"} unless defined($e{"updated"});
		if ($Woclema::Config::feed_content_length != 0) {
			my $txt = &Woclema::HTML::shorten($orig{'text'},
					$Woclema::Config::feed_content_length);
			$e{"content"} = $txt;

			# Guess content type
			if ($txt =~ /<\//s) {
				$e{'content-type'} = "html";
			} else {
				$e{'content-type'} = "text";
			}
		}
		push(@e_list, \%e);
	}

	print <<EOF;
Content-Type: application/atom+xml; charset=utf-8

EOF
	my @now = gmtime();
	print &Woclema::Atom::sprint(\@now, @e_list);
}


&main();
